#ifndef __ISOData_Country_h__
#define __ISOData_Country_h__

#include <ISOData/ISOData.h>

#include <ISOData/Updateable.h>

#include <map>

namespace ISOData
{
	class CountryName;
	class CountryNameMap;
	class CountryLanguage;
	class CountryLanguageMap;
	class SubdivisionCategory;
	class SubdivisionCategoryPtrMap;
	class Subdivision;
	class SubdivisionPtrMap;
	class Territory;
	class TerritoryPtrMap;

	class ISOData_Export Country : public Updateable
	{
		public:
			class ISOData_Export Key : public Base::Key
			{
				private:
					std::string _strCountry;

				public:
					Key();
					Key( const std::string& strCountry );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& country() const;

					virtual std::string str() const;
					static Key fromText( const std::string& strText ) throw( ISOData::Exception );
			};

			enum Status { statOfficiallyAssigned = 'A',
						  statFormerlyUsed = 'F',
						  statIndeterminatelyReserved = 'I',
						  statTransitionallyReserved = 'T',
						  statExceptionallyReserved = 'X' };

		private:
			Key _key;
			std::string _strCountryA3;
			int _nCountryN3;
			bool _bIndependent;
			Status _status;
			std::string _strShortNameEn;
			std::string _strShortNameEnu;
			std::string _strFullNameEn;
			CountryNameMap* _pnameS;
			CountryLanguageMap* _planguageS;
			SubdivisionCategoryPtrMap* _psubdivisionCategoryS;
			SubdivisionPtrMap* _psubdivisionS;
			TerritoryPtrMap* _pterritoryS;

		public:
			Country();
			Country( const Country::Key& key, const Updateable& updateable = Updateable() );
			Country( const Country& country );
			virtual ~Country();

			Country& operator=( const Country& country );
			bool operator==( const Country& country ) const;

			const Key& key() const;
			const std::string& countryA3() const;
			int countryN3() const;
			bool independent() const;
			Status status() const;
			const std::string& shortNameEn() const;
			const std::string& shortNameEnu() const;
			const std::string& fullNameEn() const;

			const CountryNameMap& names() const;
			const CountryLanguageMap& languages() const;
			const SubdivisionCategoryPtrMap& subdivisionCategorys() const;
			const SubdivisionPtrMap& subdivisions() const;
			const TerritoryPtrMap& territorys() const;

			CountryNameMap& names();
			CountryLanguageMap& languages();
			SubdivisionCategoryPtrMap& subdivisionCategorys();
			SubdivisionPtrMap& subdivisions();
			TerritoryPtrMap& territorys();

			void setCountryA3( const std::string& strCountryA3 );
			void setCountryN3( int nCountryN3 );
			void setIndependent( bool bIndependent );
			void setStatus( Status status );
			void setShortNameEn( const std::string& strShortNameEn );
			void setShortNameEnu( const std::string& strShortNameEnu );
			void setFullNameEn( const std::string& strFullNameEn );

			void addName( const CountryName& name );
			void addLanguage( const CountryLanguage& language );
			void addSubdivisionCategory( SubdivisionCategory& category );
			void addSubdivision( Subdivision& subdivision );
			void addTerritory( Territory& territory );

			static Country fromRow( const RfcDal::DalDatabase::Row& row ) throw( ISOData::Exception );
			static Country create( const Country& temp ) throw( ISOData::Exception );
			virtual void update() throw( ISOData::Exception );

			static Status statusFromText( const std::string& strStatus ) throw( ISOData::Exception );
			static Status statusFromISOText( const std::string& strStatus ) throw( ISOData::Exception );
			static std::string statusToText( Status status ) throw( ISOData::Exception );
	};

	class ISOData_Export CountryMap : public std::map<Country::Key, Country>
	{
		public:
			static CountryMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( ISOData::Exception );
	};

	class ISOData_Export CountryPtrMap : public std::map<Country::Key, Country*>
	{
		public:
			CountryPtrMap();
			CountryPtrMap( CountryMap& countryS );

			CountryPtrMap filter( int nStatusS );
	};
}

#endif /* !__ISOData_Country_h__ */
