#ifndef __ISOData_CountryLanguage_h__
#define __ISOData_CountryLanguage_h__

#include <ISOData/ISOData.h>

#include <ISOData/Country.h>
#include <ISOData/Language.h>

#include <map>

namespace ISOData
{
	class ISOData_Export CountryLanguage : public Updateable
	{
		public:
			class ISOData_Export Key : public Country::Key, public Language::Key
			{
				public:
					Key();
					Key( const Country::Key& kCountry, const Language::Key& kLanguage );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					virtual std::string str() const;
					static Key fromText( const std::string& strText ) throw( ISOData::Exception );
			};

		private:
			Key _key;
			bool _bAdministrative;
			std::string _strNotes;

		public:
			CountryLanguage();
			CountryLanguage( const CountryLanguage::Key& key, const Updateable& updateable = Updateable() );
			CountryLanguage( const CountryLanguage& language );
			virtual ~CountryLanguage();

			CountryLanguage& operator=( const CountryLanguage& language );
			bool operator==( const CountryLanguage& language ) const;

			const Key& key() const;
			bool administrative() const;
			const std::string& notes() const;

			Country& country() const throw( ISOData::Exception );
			Language& language() const throw( ISOData::Exception );

			void setAdministrative( bool bAdministrative );
			void setNotes( const std::string& strNotes );

			static CountryLanguage fromRow( const RfcDal::DalDatabase::Row& row ) throw( ISOData::Exception );
			static CountryLanguage create( const CountryLanguage& temp ) throw( ISOData::Exception );
			virtual void update() throw( ISOData::Exception );
	};

	class ISOData_Export CountryLanguageMap : public std::map<CountryLanguage::Key, CountryLanguage>
	{
		public:
			static CountryLanguageMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( ISOData::Exception );
	};
}

#endif /* !__ISOData_CountryLanguage_h__ */
