#ifndef __ISOData_CountryName_h__
#define __ISOData_CountryName_h__

#include <ISOData/ISOData.h>

#include <ISOData/Updateable.h>
#include <ISOData/Country.h>
#include <ISOData/Language.h>

#include <map>

namespace ISOData
{
	class ISOData_Export CountryName : public Updateable
	{
		public:
			class ISOData_Export Key : public Country::Key, public Language::Key
			{
				public:
					Key();
					Key( const Country::Key& kCountry, const Language::Key& kLanguage );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					virtual std::string str() const;
					static Key fromText( const std::string& strText ) throw( ISOData::Exception );
			};

		private:
			Key _key;
			std::string _strShortName;
			std::string _strShortNameU;
			std::string _strFullName;

		public:
			CountryName();
			CountryName( const CountryName::Key& key, const Updateable& updateable = Updateable() );
			CountryName( const CountryName& name );
			virtual ~CountryName();

			CountryName& operator=( const CountryName& name );
			bool operator==( const CountryName& name ) const;

			const Key& key() const;
			const std::string& shortName() const;
			const std::string& shortNameU() const;
			const std::string& fullName() const;

			Country& country() const throw( ISOData::Exception );
			Language& language() const throw( ISOData::Exception );

			void setShortName( const std::string& strShortName );
			void setShortNameU( const std::string& strShortNameU );
			void setFullName( const std::string& strFullName );

			static CountryName fromRow( const RfcDal::DalDatabase::Row& row ) throw( ISOData::Exception );
			static CountryName create( const CountryName& temp ) throw( ISOData::Exception );
			virtual void update() throw( ISOData::Exception );
	};

	class ISOData_Export CountryNameMap : public std::map<CountryName::Key, CountryName>
	{
		public:
			static CountryNameMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( ISOData::Exception );
	};
}

#endif /* !__ISOData_CountryName_h__ */
