#ifndef __ISOData_Exception_h__
#define __ISOData_Exception_h__

#include <ISOData/ISOData.h>

#include <Base4c/Exception.h>

#include <string>
#include <vector>

namespace Rfc
{
	class XMLNode;
}

namespace ISOData
{
	const int ISX_INVALIDLINE = -1;

	static const int ISX_COMPOUNDEXCEPTION = Base4c::B4CX_DERIVEDBASE;
	static const int ISX_ALIENEXCEPTION = Base4c::B4CX_DERIVEDBASE + 1;
	static const int ISX_DALEXCEPTION = Base4c::B4CX_DERIVEDBASE + 2;
	static const int ISX_AUTHEXCEPTION = Base4c::B4CX_DERIVEDBASE + 3;
	static const int ISX_NOCONNECTSTRING = Base4c::B4CX_DERIVEDBASE + 10;
	static const int ISX_FACTORYUNINITIALIZED = Base4c::B4CX_DERIVEDBASE + 15;
	static const int ISX_FILEOPENFAILED = Base4c::B4CX_DERIVEDBASE + 20;
	static const int ISX_MALFORMEDXML = Base4c::B4CX_DERIVEDBASE + 30;
	static const int ISX_MALFORMEDKEYTEXT = Base4c::B4CX_DERIVEDBASE + 31;
	static const int ISX_INCOMPLETEDATASET = Base4c::B4CX_DERIVEDBASE + 50;
	static const int ISX_CREATEFAILED = Base4c::B4CX_DERIVEDBASE + 55;
	static const int ISX_RETRIEVEFAILED = Base4c::B4CX_DERIVEDBASE + 56;
	static const int ISX_UPDATEFAILED = Base4c::B4CX_DERIVEDBASE + 57;
	static const int ISX_DELETEFAILED = Base4c::B4CX_DERIVEDBASE + 58;
	static const int ISX_NOSUCHDOMAIN = Base4c::B4CX_DERIVEDBASE + 200;
	static const int ISX_NOSUCHLANGUAGE = Base4c::B4CX_DERIVEDBASE + 201;
	static const int ISX_NOSUCHCOUNTRYSTATUS = Base4c::B4CX_DERIVEDBASE + 202;
	static const int ISX_NOSUCHCOUNTRY = Base4c::B4CX_DERIVEDBASE + 203;
	static const int ISX_NOSUCHSUBDIVISIONCATEGORY = Base4c::B4CX_DERIVEDBASE + 204;
	static const int ISX_NOSUCHSUBDIVISION = Base4c::B4CX_DERIVEDBASE + 205;

	static const int ISX_NOSUCHRECORDFORMAT = Base4c::B4CX_DERIVEDBASE + 220;
	static const int ISX_NOSUCHFIELDFORMAT = Base4c::B4CX_DERIVEDBASE + 221;
	static const int ISX_NOSUCHFIELD = Base4c::B4CX_DERIVEDBASE + 222;

	static const int ISX_NODOMAINSET = Base4c::B4CX_DERIVEDBASE + 400;
	static const int ISX_NOLANGUAGESET = Base4c::B4CX_DERIVEDBASE + 401;
	static const int ISX_NOCOUNTRYSET = Base4c::B4CX_DERIVEDBASE + 403;
	static const int ISX_NOSUBDIVISIONCATEGORYSET = Base4c::B4CX_DERIVEDBASE + 404;
	static const int ISX_NOSUBDIVISIONSET = Base4c::B4CX_DERIVEDBASE + 405;

	class ISOData_Export Exception : public Base4c::Exception
	{
		public:
			Exception( const std::string& strFile, int nLine, int nErrorCode, bool bFatal = false, const std::string& strLegend = std::string() );
			Exception( const Exception& exception );
			virtual ~Exception();

			Exception& operator=( const Exception& exception );

			static std::string explainErrorCode( int nErrorCode );

			virtual void toXML( Rfc::XMLNode* pnodeParent ) const;
			static Exception fromXML( const Rfc::XMLNode* pnodeException );
	};

	class ISOData_Export ExceptionList : public std::vector<Exception>
	{
		public:
			void toXML( Rfc::XMLNode* pnodeParent ) const;
			static ExceptionList fromXML( const Rfc::XMLNode* pnodeException ) throw( ISOData::Exception );
	};

	class ISOData_Export CompoundException : public Exception
	{
		private:
			ExceptionList _exceptionS;

		public:
			CompoundException();
			CompoundException( const std::string& strFile, int nLine );
			CompoundException( const CompoundException& exception );
			virtual ~CompoundException();

			const CompoundException& operator=( const CompoundException& e );

			bool hasExceptions() const;
			const ExceptionList& exceptions() const;
			void addException( const Exception& e );
			void addExceptions( const ExceptionList& e );

			virtual void toXML( Rfc::XMLNode* pnodeParent ) const;
			static CompoundException fromXML( const Rfc::XMLNode* pnodeException );// throw( ISOData::CompoundException );
	};
}

ISOData_Export std::ostream& operator<<( std::ostream& stream, const ISOData::Exception& e );
ISOData_Export std::ostream& operator<<( std::ostream& stream, const ISOData::CompoundException& e );

#endif /* __ISOData_Exception_h__ */
