#ifndef __ISOData_ISOFactory_h__
#define __ISOData_ISOFactory_h__

#include <ISOData/ISOData.h>

#include <ISOData/Language.h>
#include <ISOData/Country.h>
#include <ISOData/SubdivisionCategory.h>
#include <ISOData/Subdivision.h>
#include <ISOData/Territory.h>

#include <ISODal/ISODB.h>

#include <Auth4cData/AuthenticatedUser.h>

namespace ISOData
{
	class ISOData_Export ISOFactory
	{
		private:
			static LanguageMap _languageS;
			static LanguagePtrMap _languageS2;
			static CountryMap _countryS;
			static SubdivisionCategoryMap _subdivisionCategoryS;
			static SubdivisionMap _subdivisionS;
			static TerritoryMap _territoryS;

		public:
			static void initialize( const Auth4cData::AuthenticatedUser& user ) throw( ISOData::Exception );
			static void initialize( const std::string& strInit ) throw( ISOData::Exception );

			static LanguageMap& languages();
			static LanguagePtrMap& languages2();
			static CountryMap& countrys();
			static SubdivisionCategoryMap& subdivisionCategorys();
			static SubdivisionMap& subdivisions();
			static TerritoryMap& territorys();

		protected:
			static void loadLanguages( ISODal::Connection& conn ) throw( ISOData::Exception );
			static void loadCountrys( ISODal::Connection& conn ) throw( ISOData::Exception );
			static void loadSubdivisionCategorys( ISODal::Connection& conn ) throw( ISOData::Exception );
			static void loadSubdivisions( ISODal::Connection& conn ) throw( ISOData::Exception );
			static void loadTerritorys( ISODal::Connection& conn ) throw( ISOData::Exception );
	};
}

#endif /* !__ISOData_ISOFactory_h__ */
