#ifndef __ISOData_Language_h__
#define __ISOData_Language_h__

#include <ISOData/ISOData.h>

#include <ISOData/Updateable.h>

#include <map>

namespace ISOData
{
	class ISOData_Export Language : public Updateable
	{
		public:
			class ISOData_Export Key : public Base::Key
			{
				private:
					std::string _strLanguage;

				public:
					Key();
					Key( const std::string& strLanguage );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& language() const;

					virtual std::string str() const;
					static Key fromText( const std::string& strText ) throw( ISOData::Exception );
			};

		private:
			Key _key;
			std::string _strLanguageA2;
			std::string _strDescription;
			std::string _strCoLanguage;

		public:
			Language();
			Language( const Language::Key& key, const Updateable& updateable = Updateable() );
			Language( const Language& language );
			virtual ~Language();

			Language& operator=( const Language& language );
			bool operator==( const Language& language ) const;

			const Key& key() const;
			Key key2() const throw( ISOData::Exception );
			const std::string& languageA2() const;
			const std::string& description() const;
			const Language& coLanguage() const throw( ISOData::Exception );

			void setLanguageA2( const std::string& strLanguageA2 );
			void setDescription( const std::string& strDescription );
			void setCoLanguage( const Language& coLanguage );

			static Language fromRow( const RfcDal::DalDatabase::Row& row ) throw( ISOData::Exception );
			static Language create( const Language& temp ) throw( ISOData::Exception );
			virtual void update() throw( ISOData::Exception );
	};

	class ISOData_Export LanguageMap : public std::map<Language::Key, Language>
	{
		public:
			static LanguageMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( ISOData::Exception );
	};

	class ISOData_Export LanguagePtrMap : public std::map<Language::Key, Language*>
	{
		public:
			LanguagePtrMap();
			LanguagePtrMap( LanguageMap& languageS );

			LanguagePtrMap filterKeyA2();
	};
}

#endif /* !__ISOData_Language_h__ */
