#ifndef __ISOData_Subdivision_h__
#define __ISOData_Subdivision_h__

#include <ISOData/ISOData.h>

#include <ISOData/Updateable.h>
#include <ISOData/Country.h>
#include <ISOData/SubdivisionCategory.h>

#include <map>

namespace ISOData
{
	class SubdivisionName;
	class SubdivisionNameMap;
	class SubdivisionPtrMap;

	class ISOData_Export Subdivision : public Updateable
	{
		public:
			class ISOData_Export Key : public Country::Key
			{
				private:
					std::string _strSubdivision;

				public:
					Key();
					Key( const Country::Key& kCountry, const std::string& strSubdivision );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& subdivision() const;

					virtual std::string str() const;
					static Key fromText( const std::string& strText ) throw( ISOData::Exception );
			};

		public:
			// The sole purpose for orphan is removal of a child's parent
			// Using orphan in any other way will cause SIGSEGV as orphan is a dereferenced NULL pointer
			static const Subdivision& orphan;

		private:
			Key _key;
			const SubdivisionCategory* _pcategory;
			Subdivision* _pparent;
			std::string _strFootnote;
			SubdivisionNameMap* _pnameS;
			SubdivisionPtrMap* _pchildren;

		public:
			Subdivision();
			Subdivision( const Subdivision::Key& key, const Updateable& updateable = Updateable() );
			Subdivision( const Subdivision& category );
			virtual ~Subdivision();

			Subdivision& operator=( const Subdivision& category );
			bool operator==( const Subdivision& category ) const;

			const Key& key() const;
			const SubdivisionCategory& subdivisionCategory() const throw( ISOData::Exception );
			bool hasParent() const;
			const Subdivision& parent() const throw( ISOData::Exception );
			const std::string& footnote() const;

			const SubdivisionNameMap& names() const;
			const SubdivisionPtrMap& children() const;

			SubdivisionNameMap& names();
			SubdivisionPtrMap& children();
			Country& country() const throw( ISOData::Exception );

			void setSubdivisionCategory( const SubdivisionCategory& category );
			void setParent( Subdivision& parent );
			void setFootnote( const std::string& strFootnote );

			void addName( const SubdivisionName& name );

			static Subdivision fromRow( const RfcDal::DalDatabase::Row& row ) throw( ISOData::Exception );
			static Subdivision create( const Subdivision& temp ) throw( ISOData::Exception );
			virtual void update() throw( ISOData::Exception );
	};

	class ISOData_Export SubdivisionMap : public std::map<Subdivision::Key, Subdivision>
	{
		public:
			static SubdivisionMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( ISOData::Exception );
			void reparent( const RfcDal::DalDatabase::Result& result ) throw( ISOData::Exception );
	};

	class ISOData_Export SubdivisionPtrMap : public std::map<Subdivision::Key, Subdivision*>
	{
	};
}

#endif /* !__ISOData_Subdivision_h__ */
