#ifndef __ISOData_SubdivisionCategory_h__
#define __ISOData_SubdivisionCategory_h__

#include <ISOData/ISOData.h>

#include <ISOData/Updateable.h>
#include <ISOData/Country.h>

#include <map>

namespace ISOData
{
	class SubdivisionCategoryName;
	class SubdivisionCategoryNameMap;

	class ISOData_Export SubdivisionCategory : public Updateable
	{
		public:
			class ISOData_Export Key : public Country::Key
			{
				private:
					int _nSubdivisionCategory;

				public:
					Key();
					Key( const Country::Key& kCountry, int nSubdivisionCategory );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					int subdivisionCategory() const;

					virtual std::string str() const;
					static Key fromText( const std::string& strText ) throw( ISOData::Exception );
			};

		private:
			Key _key;
			SubdivisionCategoryNameMap* _pnameS;

		public:
			SubdivisionCategory();
			SubdivisionCategory( const SubdivisionCategory::Key& key, const Updateable& updateable = Updateable() );
			SubdivisionCategory( const SubdivisionCategory& category );
			virtual ~SubdivisionCategory();

			SubdivisionCategory& operator=( const SubdivisionCategory& category );
			bool operator==( const SubdivisionCategory& category ) const;

			const Key& key() const;

			const SubdivisionCategoryNameMap& names() const;

			SubdivisionCategoryNameMap& names();

			Country& country() const throw( ISOData::Exception );

			void addName( const SubdivisionCategoryName& name );

			static SubdivisionCategory fromRow( const RfcDal::DalDatabase::Row& row ) throw( ISOData::Exception );
			static SubdivisionCategory create( const SubdivisionCategory& temp ) throw( ISOData::Exception );
			virtual void update() throw( ISOData::Exception );
	};

	class ISOData_Export SubdivisionCategoryMap : public std::map<SubdivisionCategory::Key, SubdivisionCategory>
	{
		public:
			static SubdivisionCategoryMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( ISOData::Exception );
	};

	class ISOData_Export SubdivisionCategoryPtrMap : public std::map<SubdivisionCategory::Key, SubdivisionCategory*>
	{
	};
}

#endif /* !__ISOData_SubdivisionCategory_h__ */
