#ifndef __ISOData_SubdivisionCategoryName_h__
#define __ISOData_SubdivisionCategoryName_h__

#include <ISOData/ISOData.h>

#include <ISOData/Updateable.h>
#include <ISOData/SubdivisionCategory.h>
#include <ISOData/Language.h>

#include <map>

namespace ISOData
{
	class ISOData_Export SubdivisionCategoryName : public Updateable
	{
		public:
			class ISOData_Export Key : public SubdivisionCategory::Key, public Language::Key
			{
				public:
					Key();
					Key( const SubdivisionCategory::Key& kSubdivisionCategory, const Language::Key& kLanguage );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					virtual std::string str() const;
					static Key fromText( const std::string& strText ) throw( ISOData::Exception );
			};

		private:
			Key _key;
			std::string _strName;
			std::string _strNamePlural;

		public:
			SubdivisionCategoryName();
			SubdivisionCategoryName( const SubdivisionCategoryName::Key& key, const Updateable& updateable = Updateable() );
			SubdivisionCategoryName( const SubdivisionCategoryName& name );
			virtual ~SubdivisionCategoryName();

			SubdivisionCategoryName& operator=( const SubdivisionCategoryName& name );
			bool operator==( const SubdivisionCategoryName& name ) const;

			const Key& key() const;
			const std::string& name() const;
			const std::string& namePlural() const;

			SubdivisionCategory& subdivisionCategory() const throw( ISOData::Exception );
			Language& language() const throw( ISOData::Exception );

			void setName( const std::string& strName );
			void setNamePlural( const std::string& strNamePlural );

			static SubdivisionCategoryName fromRow( const RfcDal::DalDatabase::Row& row ) throw( ISOData::Exception );
			static SubdivisionCategoryName create( const SubdivisionCategoryName& temp ) throw( ISOData::Exception );
			virtual void update() throw( ISOData::Exception );
	};

	class ISOData_Export SubdivisionCategoryNameMap : public std::map<SubdivisionCategoryName::Key, SubdivisionCategoryName>
	{
		public:
			static SubdivisionCategoryNameMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( ISOData::Exception );
	};
}

#endif /* !__ISOData_SubdivisionCategoryName_h__ */
