#ifndef __ISOData_SubdivisionName_h__
#define __ISOData_SubdivisionName_h__

#include <ISOData/ISOData.h>

#include <ISOData/Updateable.h>
#include <ISOData/Subdivision.h>
#include <ISOData/Language.h>

#include <map>

namespace ISOData
{
	class ISOData_Export SubdivisionName : public Updateable
	{
		public:
			class ISOData_Export Key : public Subdivision::Key, public Language::Key
			{
				public:
					Key();
					Key( const Subdivision::Key& kSubdivision, const Language::Key& kLanguage );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					virtual std::string str() const;
					static Key fromText( const std::string& strText ) throw( ISOData::Exception );
			};

		private:
			Key _key;
			std::string _strName;
			std::string _strRomanization;

		public:
			SubdivisionName();
			SubdivisionName( const SubdivisionName::Key& key, const Updateable& updateable = Updateable() );
			SubdivisionName( const SubdivisionName& name );
			virtual ~SubdivisionName();

			SubdivisionName& operator=( const SubdivisionName& name );
			bool operator==( const SubdivisionName& name ) const;

			const Key& key() const;
			const std::string& name() const;
			const std::string& romanization() const;

			Subdivision& subdivision() const throw( ISOData::Exception );
			Language& language() const throw( ISOData::Exception );

			void setName( const std::string& strName );
			void setRomanization( const std::string& strRomanization );

			static SubdivisionName fromRow( const RfcDal::DalDatabase::Row& row ) throw( ISOData::Exception );
			static SubdivisionName create( const SubdivisionName& temp ) throw( ISOData::Exception );
			virtual void update() throw( ISOData::Exception );
	};

	class ISOData_Export SubdivisionNameMap : public std::map<SubdivisionName::Key, SubdivisionName>
	{
		public:
			static SubdivisionNameMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( ISOData::Exception );
	};
}

#endif /* !__ISOData_SubdivisionName_h__ */
