#ifndef __ISOData_Territory_h__
#define __ISOData_Territory_h__

#include <ISOData/ISOData.h>

#include <ISOData/Country.h>

#include <map>

namespace ISOData
{
	class TerritoryName;
	class TerritoryNameMap;

	class ISOData_Export Territory : public Updateable
	{
		public:
			class ISOData_Export Key : public Country::Key
			{
				private:
					int _nTerritory;

				public:
					Key();
					Key( const Country::Key& kCountry, int nTerritory );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					int territory() const;

					virtual std::string str() const;
					static Key fromText( const std::string& strText ) throw( ISOData::Exception );
			};

		private:
			Key _key;
			TerritoryNameMap* _pnameS;

		public:
			Territory();
			Territory( const Territory::Key& key, const Updateable& updateable = Updateable() );
			Territory( const Territory& category );
			virtual ~Territory();

			Territory& operator=( const Territory& category );
			bool operator==( const Territory& category ) const;

			const Key& key() const;

			const TerritoryNameMap& names() const;

			TerritoryNameMap& names();

			Country& country() const throw( ISOData::Exception );

			void addName( const TerritoryName& name );

			static Territory fromRow( const RfcDal::DalDatabase::Row& row ) throw( ISOData::Exception );
			static Territory create( const Territory& temp ) throw( ISOData::Exception );
			virtual void update() throw( ISOData::Exception );
	};

	class ISOData_Export TerritoryMap : public std::map<Territory::Key, Territory>
	{
		public:
			static TerritoryMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( ISOData::Exception );
	};

	class ISOData_Export TerritoryPtrMap : public std::map<Territory::Key, Territory*>
	{
	};
}

#endif /* !__ISOData_Territory_h__ */
