#ifndef __ISOData_TerritoryName_h__
#define __ISOData_TerritoryName_h__

#include <ISOData/ISOData.h>

#include <ISOData/Territory.h>
#include <ISOData/Language.h>

#include <map>

namespace ISOData
{
	class ISOData_Export TerritoryName : public Updateable
	{
		public:
			class ISOData_Export Key : public Territory::Key, public Language::Key
			{
				public:
					Key();
					Key( const Territory::Key& kTerritory, const Language::Key& kLanguage );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					virtual std::string str() const;
					static Key fromText( const std::string& strText ) throw( ISOData::Exception );
			};
	private:
		Key _key;
		std::string _strName;

	public:
		TerritoryName();
		TerritoryName( const TerritoryName::Key& key, const Updateable& updateable = Updateable() );
		TerritoryName( const TerritoryName& name );
		virtual ~TerritoryName();

		TerritoryName& operator=( const TerritoryName& name );
		bool operator==( const TerritoryName& name ) const;

		const Key& key() const;
		const std::string& name() const;

		Territory& territory() const throw( ISOData::Exception );
		Language& language() const throw( ISOData::Exception );

		void setName( const std::string& strName );

		static TerritoryName fromRow( const RfcDal::DalDatabase::Row& row ) throw( ISOData::Exception );
		static TerritoryName create( const TerritoryName& temp ) throw( ISOData::Exception );
		virtual void update() throw( ISOData::Exception );
	};

	class ISOData_Export TerritoryNameMap : public std::map<TerritoryName::Key, TerritoryName>
	{
		public:
			static TerritoryNameMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( ISOData::Exception );
	};
}

#endif /* !__ISOData_TerritoryName_h__ */
