#ifndef __ISOImport_ImportException_h__
#define __ISOImport_ImportException_h__

#include <ISOImport/ISOImport.h>

#include <ISOData/Exception.h>

namespace Rfc
{
	class XMLNode;
}

namespace ISOImport
{
	class ISOImport_Export ImportException : public ISOData::Exception
	{
		private:
			std::string _strSourceFile;
			int _nSourceLine;
			std::string _strData;

		public:
			ImportException( const std::string& strFile, int nLine, int nErrorCode, const std::string& strSourceFile, int nSourceLine, const std::string& strData, const std::string& strLegend = std::string() );
			ImportException( const ISOData::Exception& exception, const std::string& strSourceFile, int nSourceLine, const std::string& strData );
			ImportException( const ImportException& exception );
			virtual ~ImportException();

			ImportException& operator=( const ImportException& exception );

			const std::string& sourceFile() const;
			int sourceLine() const;
			const std::string& data() const;

			virtual void toXML( Rfc::XMLNode* pnodeParent ) const;
			static ImportException fromXML( const Rfc::XMLNode* pnodeException );
	};

	class ISOImport_Export ExceptionList : public std::vector<ImportException>
	{
		public:
			void toXML( Rfc::XMLNode* pnodeParent ) const;
			static ExceptionList fromXML( const Rfc::XMLNode* pnodeException ) throw( ISOData::Exception );
	};

	class ISOImport_Export CompoundException : public ISOData::Exception
	{
		private:
			ExceptionList _exceptionS;

		public:
			CompoundException();
			CompoundException( const std::string& strFile, int nLine );
			CompoundException( const CompoundException& exception );
			virtual ~CompoundException();

			const CompoundException& operator=( const CompoundException& e );

			bool hasExceptions() const;
			const ExceptionList& exceptions() const;
			void addException( const ImportException& e );
			void addExceptions( const ExceptionList& e );

			virtual void toXML( Rfc::XMLNode* pnodeParent ) const;
			static CompoundException fromXML( const Rfc::XMLNode* pnodeException );// throw( ISOData::CompoundException );
	};
}

ISOImport_Export std::ostream& operator<<( std::ostream& stream, const ISOImport::ImportException& e );
ISOImport_Export std::ostream& operator<<( std::ostream& stream, const ISOImport::CompoundException& e );

#endif /* !__ISOImport_ImportException_h__ */
