#ifndef __ISOImport_RecordFormat_h__
#define __ISOImport_RecordFormat_h__

#include <ISOImport/ISOImport.h>

#include <ISOData/Base.h>
#include <ISOData/Exception.h>

#include <ISODal/ISODB.h>

#include <librfcdal/DalDatabase.h>

namespace ISOImport
{
	class ISOImport_Export RecordFormat : public ISOData::Base
	{
		public:
			class ISOImport_Export Key : public ISOData::Base::Key
			{
				public:
					static const int invalidRecordFormat;

				private:
					int _nRecordFormat;

				public:
					Key();
					Key( int nRecordFormat );
					Key( const Key& key );

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					int record() const;

					virtual std::string str() const;
			};

			class ISOImport_Export Ident : public ISOData::Base::Key
			{
				private:
					std::string _strDescription;

				public:
					Ident();
					Ident( const std::string& strDescription );
					Ident( const Ident& ident );

					Ident& operator=( const Ident& ident );
					bool operator==( const Ident& ident ) const;
					bool operator!=( const Ident& ident ) const;
					bool operator<( const Ident& ident ) const;

					const std::string& description() const;

					virtual std::string str() const;
			};

			class ISOImport_Export Field : public ISOData::Base
			{
				public:
					class ISOImport_Export Key : public RecordFormat::Key
					{
						public:
							static const int invalidField;

						private:
							int _nField;

						public:
							Key();
							Key( const RecordFormat::Key& kRecordFormat, int nField );
							Key( const Key& key );

							Key& operator=( const Key& key );
							bool operator==( const Key& key ) const;
							bool operator!=( const Key& key ) const;
							bool operator<( const Key& key ) const;

							int field() const;

							virtual std::string str() const;
					};

					class ISOImport_Export Ident : public ISOData::Base::Key
					{
						private:
							std::string _strDescription;

						public:
							Ident();
							Ident( const std::string& strDescription );
							Ident( const Ident& ident );

							Ident& operator=( const Ident& ident );
							bool operator==( const Ident& ident ) const;
							bool operator!=( const Ident& ident ) const;
							bool operator<( const Ident& ident ) const;

							const std::string& description() const;

							virtual std::string str() const;
					};

				private:
					Key _key;
					Ident _ident;

				public:
					Field();
					Field( const Field::Key& key );
					Field( const Field& field );
					virtual ~Field();

					Field& operator=( const Field& field );

					const Key& key() const;
					const Ident& ident() const;

					static Field fromRow( const RfcDal::DalDatabase::Row& row, ISODal::Connection& conn ) throw( ISOData::Exception );
			};

			class ISOImport_Export FieldMap : public std::map<Field::Key, Field>
			{
				public:
					static FieldMap fromResult( const RfcDal::DalDatabase::Result& result, ISODal::Connection& conn ) throw( ISOData::Exception );
			};

			class ISOImport_Export FieldRPtrMap : public std::map<Field::Ident, Field*>
			{
				public:
					FieldRPtrMap();
					FieldRPtrMap( FieldMap& fieldS );
			};

		private:
			Key _key;
			Ident _ident;
			std::string _strDelimiter;
			FieldMap _fieldS;
			FieldRPtrMap _rfieldS;

		public:
			RecordFormat();
			RecordFormat( const RecordFormat::Key& key );
			RecordFormat( const RecordFormat& record );
			virtual ~RecordFormat();

			RecordFormat& operator=( const RecordFormat& record );

			const Key& key() const;
			const Ident& ident() const;
			const std::string& delimiter() const;
			const Field& field( const RecordFormat::Field::Ident& id ) const throw( ISOData::Exception );

			static RecordFormat fromRow( const RfcDal::DalDatabase::Row& row, ISODal::Connection& conn ) throw( ISOData::Exception );
			void reindex();
	};

	class ISOImport_Export RecordFormatMap : public std::map<RecordFormat::Key, RecordFormat>
	{
		public:
			static RecordFormatMap fromResult( const RfcDal::DalDatabase::Result& result, ISODal::Connection& conn ) throw( ISOData::Exception );
			void reindex();
	};

	class ISOImport_Export RecordFormatRPtrMap : public std::map<RecordFormat::Ident, RecordFormat*>
	{
		public:
			RecordFormatRPtrMap();
			RecordFormatRPtrMap( RecordFormatMap& recordS );
	};

	class ISOImport_Export RecordFormatBroker
	{
		private:
			static bool _bInitialized;
			static RecordFormatMap _recordS;
			static RecordFormatRPtrMap _rrecordS;

		public:
			static bool isInitialized();
			static void initialize( ISODal::Connection& conn ) throw( ISOData::Exception );

			static const RecordFormat& recordFormat( const RecordFormat::Ident& id ) throw( ISOData::Exception );
	};
}

#endif /* !__ISOData_RecordFormat_h__ */
